@@ -5,7 +5,7 @@ __all__ = [
     "ValidationError",
     "alias",
     "check_types",
-    "conversion",
+    "conversions",
     "deserialization",
     "deserialize",
     "deserializer",
@@ -25,7 +25,7 @@ __all__ = [
 
 
 from . import (
-    conversion,
+    conversions,
     deserialization,
     fields,
     json_schema,
@@ -35,7 +35,7 @@ from . import (
     validation,
 )
 from .aliases import alias
-from .conversion import deserializer, serializer
+from .conversions import deserializer, serializer
 from .deserialization import deserialize
 from .json_schema.refs import schema_ref
 from .json_schema.schema import schema
@@ -49,8 +49,16 @@ from .validation import (
 )
 from .visitor import Unsupported
 
-# Handle standard library + internal types
-serializer(ValidationError.format, ValidationError)
-from . import std_types  # noqa: E402
 
-del std_types  # clean namespace
+def default_conversions():
+    """Handle standard library + internal types"""
+    from typing import Sequence
+    from . import std_types  # noqa F401
+    from .validation.errors import LocalizedError
+
+    deserializer(ValidationError.deserialize, Sequence[LocalizedError], ValidationError)
+    serializer(ValidationError.serialize, ValidationError)
+
+
+default_conversions()
+del default_conversions  # clean namespace
