@@ -21,8 +21,8 @@ from typing import (  # type: ignore
 )
 
 from apischema import settings
-from apischema.conversion.utils import Conversions
-from apischema.conversion.visitor import Conv
+from apischema.conversions.utils import Conversions
+from apischema.conversions.visitor import Conv
 from apischema.dataclasses import is_dataclass, replace
 from apischema.dataclasses.cache import Field, FieldKind
 from apischema.json_schema.constraints import (
@@ -363,6 +363,21 @@ class SchemaBuilder(SchemaVisitor[Conv, Optional[Schema], JsonSchema]):
                 for res in results
             )
             result = json_schema(type=list(types))
+        elif (
+            len(results) == 2
+            and all("type" in res for res in results)
+            and {"type": "null"} in results
+        ):
+            for result in results:
+                if result != {"type": "null"}:
+                    types = result["type"]
+                    if isinstance(types, str):
+                        types = [types]  # type: ignore
+                    if "null" not in types:
+                        result = JsonSchema({**result, "type": [*types, "null"]})
+                    break
+            else:
+                raise NotImplementedError()
         else:
             result = json_schema(anyOf=results)
         return _with_schema(schema, result)
@@ -403,7 +418,7 @@ class SerializationSchemaBuilder(SerializationSchemaVisitor, SchemaBuilder):
         pass
 
 
-TypesWithConversions = Sequence[Union[AnyType, Tuple[AnyType, Conversions]]]
+TypesWithConversions = Collection[Union[AnyType, Tuple[AnyType, Conversions]]]
 
 
 def _default_version(
